# -- Main script  of LibreWeb project
# -----------------------------------
# -- author Vladimir Boscaneanu
# -- e-mail: vladboscaneanu@gmail.com
# ------------------------------------

# ----importing modules

# Standart modules

# LibreOffice modules
import uno
import unohelper
from html.parser import HTMLParser
from com.sun.star.awt import XActionListener
from com.sun.star.awt.PosSize import SIZE, POSSIZE
import urllib.request
import urllib.error

# LibreWeb modules

from  savemodule import LibreWebPickle
from messagebox import MsgBox

# ----Global variables

ctx = XSCRIPTCONTEXT.getComponentContext()
desktop = XSCRIPTCONTEXT.getDesktop()
document = XSCRIPTCONTEXT.getDocument()
dialog_title = "LibreWeb Settings"
dialog_width = 400
dialog_height = 300
message_box = MsgBox(desktop)
dialog_x = 0
dialog_y = 0
accepted_tags = ["h1", "h2", "h3", "h4", "h5", "h6", "span", "p", "a", "div"]
collected_items = []
save_file_name = "libreweb.data"

# Create an pickle instance to work with save_file

save_file = LibreWebPickle(save_file_name)


# -- Defining parser


class LibreWebParser(HTMLParser):
    def __init__(self, target_tag):
        ''' Please select a target_tag
        for your web page.
        Don't forget to use the feed() function.'''
        HTMLParser.__init__(self)
        self.targetTag = target_tag
        self.targetFound = False
        self.collectedData = []

    def handle_starttag(self, tag, attrs):
        self.targetFound = False
        if tag == self.targetTag:
            self.targetFound = True

    def handle_endtag(self, tag):
        if tag == self.targetTag:
            self.targetFound = False

    def handle_data(self, data):
        if self.targetFound:
            self.collectedData.append(data)


# ------ Defining functions


def create_istance(name):
    return ctx.getServiceManager().createInstanceWithContext(name, ctx)


def add_to_dialog(name, type, props={}):
    model = dialog_model.createInstance("com.sun.star.awt.UnoControl" + type + "Model")
    dialog_model.insertByName(name, model)
    for key, value in props.items():
        setattr(model, key, value)


def open_url(url):
    try:
        result = urllib.request.urlopen(url).read().decode("utf-8")
        return result
    except ValueError:
        message_box.show("Invalid URL", "Attention", 2)
    except urllib.error.URLError:
        message_box.show("Site not reacheble or internet connection is missing", "Attention", 2)
    except Exception as error:
        str_error = ""
        for i in error.args:
            str_error = str_error + "\n" + str(i)
        message_box.show(str_error, "Attention", 2)
    return False


# -- Listeners

# -- Check Url Button


class CheckURLListener(unohelper.Base, XActionListener):
    '''Accepts as argument the view of four objects'''

    def __init__(self, control_url, element_to_search, tag_count, result_label_control):
        self.object = control_url
        self.elementToSearch = element_to_search
        self.tagCount = tag_count
        self.resultLabel = result_label_control

    def actionPerformed(self, event):

        target_tag = (self.elementToSearch.Text).strip()
        target_url = self.object.Text.strip()
        global collected_items
        # -- clean previous items
        if collected_items:
            self.tagCount.removeItems(0, len(collected_items))
            self.tagCount.setEnable(False)
            self.resultLabel.setText("No data.")
        if target_tag in accepted_tags:
            result = open_url(target_url)
            if result:
                my_parser = LibreWebParser(target_tag)
                my_parser.feed(result)
                collected_items = my_parser.collectedData
                if collected_items:
                    self.tagCount.addItems([x for x in range(0, len(collected_items))], 0)
                    self.tagCount.setEnable(True)
        else:
            message_box.show("Not supported tag!", "Atention!", 2)

    def disposing(self, event):
        pass


# -- TagCountListBox if is modified


class TagCountListener(unohelper.Base, XActionListener):
    def __init__(self, list_box_control, result_label_control):
        self.ListBoxControl = list_box_control
        self.ResultLabelControl = result_label_control

    def actionPerformed(self, event):
        item_position = self.ListBoxControl.getSelectedItemPos()
        if item_position >= 0:
            self.ResultLabelControl.setText(collected_items[item_position])


# -- create dialog model
dialog_model = create_istance("com.sun.star.awt.UnoControlDialogModel")
dialog_model_props = {"PositionX": dialog_x, "PositionY": dialog_y,
                      "Width": dialog_width, "Height": dialog_height,
                      "Title": dialog_title}
for key, value in dialog_model_props.items():
    setattr(dialog_model, key, value)

# -- adding dialog elements
# -- Document Name
doc_title = document.getTitle
DocumentName_props = {"Border": 3, "FontHeight": 15, "PositionX": 10,
                      "PositionY": 0, "Width": 380, "Height": 20,
                      "Align": 1, "Label": doc_title(),
                      "VerticalAlign": 1}
add_to_dialog("DocumentName", "FixedText", DocumentName_props)

# URL TextBox
URLTextBox_props = {"Border": 3, "FontHeight": 15, "PositionX": 10,
                    "PositionY": 30, "Width": 380, "Height": 20}
add_to_dialog("URLTextBox", "Edit", URLTextBox_props)

# Sheet Control
SheetControl_props = {"PositionX": 0, "PositionY": 60, "Width": 200,
                      "Height": 90, "Label": "Sheet Control", "FontHeight": 10}
add_to_dialog("Sheet Control", "GroupBox", SheetControl_props)

# Tag Control
TagControl_props = {"PositionX": 200, "PositionY": 60, "Width": 200,
                    "Height": 90, "Label": "Tag Control", "FontHeight": 10}
add_to_dialog("TagControl", "GroupBox", TagControl_props)

# SheetsListBox
Sheet_name = document.getCurrentController().ActiveSheet.Name
Sheet_props = {"PositionX": 10, "PositionY": 80, "Width": 180,
               "Height": 20, "Border": 3,
               "VerticalAlign": 1, "Align": 1,
               "FontHeight": 15}
add_to_dialog("SheetName", "FixedText", Sheet_props)

# ElementToSearch
ElementToSearch_props = {"Border": 3, "FontHeight": 15, "PositionX": 210,
                         "PositionY": 80, "Width": 180, "Align": 1,
                         "Height": 20}
add_to_dialog("ElementToSearch", "Edit", ElementToSearch_props)

# ColumnListBox
Columns_count = document.CurrentController.ActiveSheet.Columns.ElementNames[:20]
Columns_props = {"Border": 3, "FontHeight": 15, "PositionX": 10,
                 "PositionY": 110, "Width": 80, "Height": 20, "Align": 1,
                 "Dropdown": True, "StringItemList": Columns_count}
add_to_dialog("ColumnListBox", "ListBox", Columns_props)

# RowListBox
Rows_count = [x for x in range(1, 21)]
Rows_props = {"Border": 3, "FontHeight": 15, "PositionX": 110,
              "PositionY": 110, "Width": 80, "Height": 20, "Dropdown": True,
              "Align": 1, "StringItemList": Rows_count}
add_to_dialog("RowListBox", "ListBox", Rows_props)

# TagCountListBox
TagCount_props = {"Border": 3, "Align": 1, "Dropdown": True, "FontHeight": 15,
                  "PositionX": 210, "PositionY": 110, "Width": 180,
                  "Height": 20, "Enabled": False}
add_to_dialog("TagCountListBox", "ListBox", TagCount_props)

# TagResult
TagResult_props = {"Label": "Tag Result", "FontHeight": 10, "PositionX": 0,
                   "PositionY": 160, "Width": 400, "Height": 80}
add_to_dialog("TagResult", "GroupBox", TagResult_props)

# ResultLabel
ResultLabel_props = {"Label": "No data.", "Align": 1, "FontHeight": 15, "Border": 3,
                     "Width": 380, "Height": 60, "PositionX": 10, "PositionY": 170,
                     "VerticalAlign": 1, "MultiLine": True}
add_to_dialog("ResultLabel", "FixedText", ResultLabel_props)

# Control Panel
ControlPanel_props = {"Label": "Control Panel", "FontHeight": 10, "PositionX": 0,
                      "PositionY": 250, "Width": 400, "Height": 50}
add_to_dialog("ControlPanel", "GroupBox", ControlPanel_props)

# CheckURLButton
CheckURL_props = {"Label": "Check URL", "Align": 1, "FontHeight": 15, "Width": 80,
                  "Height": 20, "PositionX": 10, "PositionY": 270,
                  "VerticalAlign": 1}
add_to_dialog("CheckURLButton", "Button", CheckURL_props)

# SaveAs
SaveAs_props = {"Label": "Save as:", "FontHeight": 15, "Align": 1, "Border": 0,
                "Width": 50, "Height": 20, "PositionX": 110, "PositionY": 270,
                "VerticalAlign": 1}
add_to_dialog("SaveAs", "FixedText", SaveAs_props)

# DataType
DataType_props = {"Align": 1, "Dropdown": True, "Width": 120, "Height": 20,
                  "PositionX": 170, "PositionY": 270, "Border": 3, "FontHeight": 15}
add_to_dialog("DataTypeListBox", "ListBox", DataType_props)

# SaveButton
SaveButton_props = {"Align": 1, "Label": "Save", "FontHeight": 15, "Width": 80,
                    "Height": 20, "PositionX": 310, "PositionY": 270,
                    "VerticalAlign": 1, "Enabled": False}
add_to_dialog("SaveButton", "Button", SaveButton_props)

# -- create a dialog control

dialog = create_istance("com.sun.star.awt.UnoControlDialog")
dialog.setModel(dialog_model)

# -- get the items control
URLTextBox_control = dialog.getControl("URLTextBox")
element_to_search = dialog.getControl("ElementToSearch")
TagCount_control = dialog.getControl("TagCountListBox")
CheckURL_control = dialog.getControl("CheckURLButton")
ResultLabel_control = dialog.getControl("ResultLabel")

# -- add listeners
CheckURL_control.addActionListener(CheckURLListener(URLTextBox_control, element_to_search,
                                                    TagCount_control, ResultLabel_control))
TagCount_control.addActionListener(TagCountListener(TagCount_control, ResultLabel_control))
# -- show dialog

toolkit = create_istance("com.sun.star.awt.ExtToolkit")
dialog.setVisible(False)
dialog.createPeer(toolkit, None)


# -- Main function


def Settings(*args):
    # add some information at the runtime moment

    # -- Current sheet's name
    Sheet_name = document.getCurrentController().ActiveSheet.Name
    dialog_model.getByName("SheetName").Label = Sheet_name

    # -- run dialog
    dialog.execute()


# End of script
g_ExportedScripts = Settings
